package com.ltdigital.sdk.android.demo.activity;

import android.Manifest;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.graphics.Paint;
import android.graphics.Point;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.view.SurfaceHolder;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.android.core.log.Log;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.Binarizer;
import com.google.zxing.BinaryBitmap;
import com.google.zxing.ChecksumException;
import com.google.zxing.DecodeHintType;
import com.google.zxing.FormatException;
import com.google.zxing.LuminanceSource;
import com.google.zxing.NotFoundException;
import com.google.zxing.PlanarYUVLuminanceSource;
import com.google.zxing.RGBLuminanceSource;
import com.google.zxing.Result;
import com.google.zxing.common.HybridBinarizer;
import com.google.zxing.qrcode.QRCodeReader;
import com.ltdigital.sdk.android.demo.zxing.MySurfaceView;
import com.ltdigital.sdk.android.demo.zxing.camera.CameraManager;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;


public abstract class BaseQRCodeScanActivity extends BaseVcActivity implements SurfaceHolder.Callback, Handler.Callback {

    private CameraManager cameraManager;
    private boolean hasSurface;
    private Handler decodeHandler;
    private HandlerThread decodeHandlerThread;
    private final int MSG_CODE_DECODE = 1;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        setContentView(getLayout());
        super.onCreate(savedInstanceState);
        SurfaceHolder holder = getSurfaceView().getHolder();
        holder.addCallback(this);
    }

    @Override
    protected void initView() {
        decodeHandlerThread = new HandlerThread("decodeHandlerThread");
        decodeHandlerThread.start();
        decodeHandler = new Handler(decodeHandlerThread.getLooper(), this);

        hasSurface = false;
        cameraManager = new CameraManager(getApplication());
    }

    @Override
    protected void initTitle() {
        this.isLightMode = false;
    }

    @Override
    protected boolean checkPermission(String[] strings, CharSequence charSequence, int i) {
        return false;
    }

    @Override
    protected void onTimeChange(int i) {

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.e(TAG, "onResume");
        getRxPermission().request(Manifest.permission.CAMERA)
                .subscribe(granted -> {
                    if (granted) {
                        Log.e(TAG, "hasSurface:" + hasSurface);
                        if (hasSurface) {
                            // The activity was paused but not stopped, so the surface still exists. Therefore
                            // surfaceCreated() won't be called, so init the camera here.
                            initCamera(getSurfaceView().getHolder());
                        }
                    } else {
                        showToast("无法获取摄像头权限");
                        finish();
                    }
                });
    }

    @Override
    protected void onPause() {
        if (cameraManager != null) {
            cameraManager.stopPreview();
            cameraManager.closeDriver();
        }
        if (decodeHandler != null) {
            decodeHandler.removeMessages(MSG_CODE_DECODE);
        }
        //historyManager = null; // Keep for onActivityResult
        if (!hasSurface) {
            SurfaceHolder holder = getSurfaceView().getHolder();
            holder.removeCallback(this);
        }
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (decodeHandler != null) {
            decodeHandler.removeCallbacksAndMessages(null);
        }

        if (decodeHandlerThread != null && decodeHandlerThread.getLooper() != null) {
            decodeHandlerThread.getLooper().quitSafely();
        }
    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        if (!hasSurface) {
            hasSurface = true;
            initCamera(holder);
        }
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        Point resolution = cameraManager.getCameraResolution();
        if (resolution != null) {
            //获取当前配置的相机预览像素
            int cameraW = resolution.y;
            int cameraH = resolution.x;
            int destW;
            int destH;

            float oScale = (float) cameraW / cameraH;
            float dScale = (float) width / height;

            if (oScale > dScale) {
                //宽大于高
                destW = width;
                destH = (int) (destW / oScale);
            } else {
                //高大于宽
                destH = height;
                destW = (int) (destH * oScale);
            }

            getSurfaceView().resize(destW, destH);
        }
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        hasSurface = false;
    }

    private LuminanceSource getSourceFromBitmap(Bitmap bitmap) {
        int width = bitmap.getWidth();
        int height = bitmap.getHeight();
        int[] pixels = new int[width * height];
        bitmap.getPixels(pixels, 0, width, 0, 0, width, height);
        return new RGBLuminanceSource(width, height, pixels);
    }

    /**
     * 解析传入的Bitmap
     *
     * @param bitmap
     */
    private Result decode(Bitmap bitmap) {
        return decode(getSourceFromBitmap(bitmap));
    }

    /**
     * 解析传入的Bitmap
     *
     * @param source
     */
    private Result decode(LuminanceSource source) {
        return decode(new HybridBinarizer(source));
    }

    private Result decode(Binarizer binarizer) {
        try {
            BinaryBitmap binaryBitmap = new BinaryBitmap(binarizer);
            Map<DecodeHintType, Object> hints = new HashMap<>();
            hints.put(DecodeHintType.CHARACTER_SET, "utf-8");
            hints.put(DecodeHintType.TRY_HARDER, Boolean.TRUE);
            hints.put(DecodeHintType.POSSIBLE_FORMATS, BarcodeFormat.QR_CODE);

            return new QRCodeReader().decode(binaryBitmap, hints);
        } catch (NotFoundException | ChecksumException | FormatException e) {
//            e.printStackTrace();
//            Log.e(TAG, "error" + e);
            return null;
        }
    }

    public Bitmap toGrayscale(Bitmap bmpOriginal) {
        int width, height;
        height = bmpOriginal.getHeight();
        width = bmpOriginal.getWidth();

        Bitmap bmpGrayscale = Bitmap.createBitmap(width, height, Bitmap.Config.RGB_565);
        Canvas c = new Canvas(bmpGrayscale);
        Paint paint = new Paint();
        ColorMatrix cm = new ColorMatrix();
        //Set the matrix to affect the saturation of colors.
        //A value of 0 maps the color to gray-scale.
        cm.setSaturation(0);
        ColorMatrixColorFilter f = new ColorMatrixColorFilter(cm);
        paint.setColorFilter(f);
        c.drawBitmap(bmpOriginal, 0, 0, paint);
        return bmpGrayscale;
    }

    private void initCamera(SurfaceHolder surfaceHolder) {
        Log.d(TAG, "initCamera");
        if (surfaceHolder == null) {
            throw new IllegalStateException("No SurfaceHolder provided");
        }
        if (cameraManager.isOpen()) {
            Log.d(TAG, "initCamera() while already open -- late SurfaceView callback?");
            return;
        }
        try {
            cameraManager.openDriver(surfaceHolder);
            cameraManager.startPreview();
            restartPreviewAndDecode();
            // Creating the handler starts the preview, which can also throw a RuntimeException.
        } catch (IOException ioe) {
            Log.e(TAG, "initCamera fail", ioe);
        } catch (RuntimeException e) {
            // Barcode Scanner has seen crashes in the wild of this variety:
            // java.?lang.?RuntimeException: Fail to connect to camera service
            Log.e(TAG, "Unexpected error initializing camera", e);
        }
    }


    private void restartPreviewAndDecode() {
        cameraManager.requestPreviewFrame(decodeHandler, MSG_CODE_DECODE);
    }

    @Override
    public boolean handleMessage(@NonNull Message msg) {
        switch (msg.what) {
            case MSG_CODE_DECODE:
                try {
                    LuminanceSource source = handlePreviewData((byte[]) msg.obj, msg.arg1, msg.arg2);
                    if (source != null) {
                        Result result = decode(source);
                        if (result != null) {
                            Log.d(TAG, "decode success:" + result);
                            if (!dealDecodeResult(result.toString())) {
                                mainHandler.postDelayed(this::restartPreviewAndDecode, 300);
                            }
                        } else {
                            throw new Exception("decode fail result is null!");
                        }
                    } else {
                        throw new Exception("decode fail source is null!");
                    }
                } catch (Exception e) {
//                    e.printStackTrace();
                    restartPreviewAndDecode();
                }
                return true;
        }
        return false;
    }


    private LuminanceSource handlePreviewData(byte[] data, int width, int height) {
        PlanarYUVLuminanceSource source = cameraManager.buildLuminanceSource(data, width, height);
        return source;
    }

    @Override
    public void showError(String s, String s1) {

    }

    @Override
    public void startLoading() {

    }

    @Override
    public void finishLoading() {

    }

    /**
     * 二维码解析结果函数
     *
     * @param result 如果返回false则继续调用重新解码功能
     * @return
     */
    abstract boolean dealDecodeResult(String result);

    @NonNull
    abstract MySurfaceView getSurfaceView();
}
