package com.ltdigital.sdk.android.demo.bluetooth

import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCharacteristic
import android.content.Context
import android.os.Handler
import android.os.HandlerThread
import android.os.Looper
import android.util.Log
import com.android.ble.oad.Util
import com.android.core.Utils
import com.ltdigital.sdk.android.demo.bean.*
import com.ltdigital.sdk.android.demo.bluetooth.Operation.DataSyncOperationCallback.REMOTE_BUSY
import com.ltdigital.sdk.android.demo.callback.*
import com.ltdigital.sdk.android.demo.helper.*
import java.io.File
import java.io.FileInputStream
import java.io.FileNotFoundException
import java.lang.StringBuilder
import java.lang.ref.WeakReference
import java.math.BigDecimal
import java.math.RoundingMode
import java.nio.ByteBuffer
import java.util.*
import java.util.zip.CRC32
import kotlin.collections.ArrayList
import kotlin.collections.HashSet
import kotlin.experimental.and
import kotlin.math.min

class DeviceManager(val context: Context, val gatt: BluetoothGatt) :
    OperationQueue.OperationCallback {

    companion object {
        val TAG = "DeviceManager"
    }

    private val queue: OperationQueue = OperationQueue(this)
    private var heartValueCallback: HeartValueCallback? = null
    private var logCallback: LogCallback? = null
    private var testCallback: TestResultCallback? = null
    private var speedCallback: SpeedCallback? = null
    private var rthTransfer: RTHTransfer? = null
    private var verifyCallback: VerifyCallback? = null
    private val handlerThread: HandlerThread = HandlerThread(TAG)
    private var handler: Handler? = null
    private var mainHandler: Handler = Handler(Looper.getMainLooper())
    private var receivedByte: Long = 0
    private var lastSpeed: Float = 0f
    private var mtu = 0
    private var isRelease = false
    private var byteBuffer: ByteBuffer? = null
    private var logHelper: FileSaveHelper? = null
    private var logFileName: String? = null
    private var logDateTime: Long = 0
    private val gattStatusCallback: MutableSet<WeakReference<GattStatusChangeCallback>> =
        HashSet<WeakReference<GattStatusChangeCallback>>()

    init {
        handlerThread.start()
        byteBuffer = ByteBuffer.allocate(1024)
        handler = Handler(handlerThread.looper)
        handler?.post(object : Runnable {
            override fun run() {
                val speed = BigDecimal.valueOf(receivedByte / 1024.0).setScale(2, RoundingMode.DOWN)
                    .toFloat()
                if (speed != lastSpeed) {
                    speedCallback?.onSpeedChange(speed)
                    lastSpeed = speed
                }
                receivedByte = 0
                handler?.removeCallbacks(this)
                handler?.postDelayed(this, 1000)
            }
        })
    }

    fun addGattStatusChangeCallback(callback: GattStatusChangeCallback) {
        val reference = WeakReference<GattStatusChangeCallback>(callback)
        if (!gattStatusCallback.contains(reference)) {
            gattStatusCallback.add(reference)
        }
    }

    fun removeGattStatusChangeCallback(callback: GattStatusChangeCallback) {
        val reference = WeakReference<GattStatusChangeCallback>(callback)
        if (!gattStatusCallback.contains(reference)) {
            gattStatusCallback.remove(reference)
        }
    }

    fun heartTranslate(enable: Boolean) {
        if (enable) {
            rthTransfer ?: let {
                rthTransfer = RTHTransfer(
                    Utils.getCurrentTimeInSecond(), FileSaveHelper(
                        "${context.getExternalFilesDir(null).toString()}/data/", "心率实时数据_${
                            Utils.getFormatDateTimeYMDStr(context, Utils.getCurrentTimeInSecond())
                        }"
                    )
                )
            }

        } else {
            rthTransfer?.let {
                it.dataSaveHelper.close(true)
                rthTransfer = null
            }
        }
        sendCommand2(byteArrayOf(0x1A, if (enable) 0x01 else 0x00))
    }

    fun refreshMtu(mtu: Int) {
        DeviceManager@ this.mtu = mtu - 3
    }

    fun onReceiveDeviceData(
        gatt: BluetoothGatt?,
        characteristic: BluetoothGattCharacteristic,
        value: ByteArray
    ) {
        receivedByte += value.size
        characteristic.service?.uuid?.let { serviceUUID ->
            Log.d(
                TAG,
                "onReceiveDeviceData serviceUUID:$serviceUUID /// data:[${
                    Utils.byteArrayToHexStrWithSpaceSplit(value)
                }]"
            )
            if (serviceUUID == BleConstants.DEVICE_S_DATA_SERVICE_UUID && characteristic.uuid == BleConstants.DEVICE_S_DATA_NOTIFY_CHARACTERISTIC_UUID) {
                val command: Int = value[0].toInt().and(0xff)
                Log.e(TAG, "command:$command")
                when (command) {
                    0x00 -> {
                        //收到ACK
                        queue.currentOperation?.let {
                            it.getCallback()?.onSuccess()
                        }
                    }
                    0x22 -> {
                        //激活结果
                        val code = value[1].toInt().and(0xff)
                        if (code == 0x01) {
                            //获取采集器激活结果
                            val mac = StringBuilder().append(byte2HexStr(value[2])).append(":")
                                .append(byte2HexStr(value[3])).append(":")
                                .append(byte2HexStr(value[4])).append(":")
                                .append(byte2HexStr(value[5])).append(":")
                                .append(byte2HexStr(value[6])).append(":")
                                .append(byte2HexStr(value[7])).toString()
                            val deviceType = String(ByteArray(6) { value[it + 8] })
                            val activeSuccess = value[14].toInt().and(0xff) == 0x01
                            verifyCallback?.onDeviceInfo(mac, deviceType, activeSuccess)
                        } else if (code == 0x02) {
                            //采集器收到所有激活信息成功主动发送结果
                            val activeSuccess = value[2].toInt().and(0xff) == 0x01
                            verifyCallback?.onVerifyResult(activeSuccess)
                            val mac = gatt?.device?.address
                            printLog("mac:$mac -激活结果- $activeSuccess")
                        }
                    }
                    0xFE -> {
                        //测试结果
                        val code = value[1].toInt().and(0xff)
                        if (code == 0x01) {
                            testCallback?.onReceivedAutoTestResult(
                                AutoTestResult(
                                    value[2].toInt().and(0xff),
                                    value[3].toInt().and(0xff) == 0x01,
                                    value[4].toInt().and(0xff) == 0x01,
                                    value[5].toInt().and(0xff) == 0x01,
                                    value[6].toInt().and(0xff) == 0x01,
                                    value[9].toInt().and(0xff) == 0x01,
                                    value[7].toInt().and(0xff) == 0x01,
                                    value[8].toInt().and(0xff) == 0x01
                                )
                            )
                        } else if (code == 0x02) {
                            val testType = value[2].toInt().and(0xff)
                            if (testType == 0x01) {
                                //触摸测试结果反馈
                                val touchType = value[3].toInt().and(0xff)
                                testCallback?.onReceivedTouchTestResult(
                                    touchType == 0x02 || touchType == 0x03,
                                    touchType == 0x01 || touchType == 0x03
                                )
                            } else if (testType == 0x03) {
                                //充电信号结果反馈
                                testCallback?.onReceivedChargeTestResult(true)
                            } else if (testType == 0x05) {
                                //充电信号结果反馈
                                val skin = value[3].toInt().and(0xff)
                                val hr = value[4].toInt().and(0xff)
                                val hr_conf = value[5].toInt().and(0xff)
                                val spo2 = value[6].toInt().and(0xff)
                                val spo2_conf = value[7].toInt().and(0xff)
                                testCallback?.onReceivedHeartTestResult(
                                    if (skin == 1 && hr_conf >= 25) hr else null,
                                    if (skin == 1 && spo2_conf >= 25) spo2 else null
                                )
                            }
                        } else if (code == 0x04) {
                            testCallback?.onReceivedTimeout()
                        }
                    }


                }
            }
        }

    }

    private fun byte2HexStr(byte: Byte): String {
        val hexString = Integer.toHexString(byte.toInt().and(0xff))
        if (hexString.length != 2) {
            return "0$hexString".uppercase()
        } else {
            return hexString.uppercase()
        }
    }


    fun onCharacteristicWriteSuccess(uuid: UUID) {
        queue.currentOperation?.type?.let {
            if (it == Operation.TYPE_STATUS && uuid == BleConstants.DEVICE_STATUS_WRITE_CHARACTERISTIC_UUID) {
                queue.currentOperation?.callback?.onSuccess()
            }
        }
    }

    fun release() {
        for (reference in gattStatusCallback) {
            reference.get()?.onGattDisconnected()
        }
        gattStatusCallback.clear()
        isRelease = true
        handler?.removeCallbacksAndMessages(null)
        queue.removeAllOperations()
    }

    fun setLogCallback(callback: LogCallback?) {
        logCallback = callback
    }

    fun setSpeedCallback(callback: SpeedCallback?) {
        speedCallback = callback
    }

    fun setHeartValueCallback(callback: HeartValueCallback?) {
        heartValueCallback = callback
    }

    fun setTestResultCallback(callback: TestResultCallback?) {
        testCallback = callback
    }

    fun setVerifyCallback(callback: VerifyCallback?) {
        verifyCallback = callback
    }

    fun sendTestCommand(byteArray: Array<Byte>?) {
        sendTestCommand(byteArray, null)
    }

    fun sendTestCommand(byteArray: Array<Byte>?, callback: Operation.DataSyncOperationCallback?) {
        handler?.post {
            byteArray?.let {
                sendCommand2(ByteArray(it.size + 1) { index ->
                    when (index) {
                        0 -> 0xFE.toByte()
                        else -> it[index - 1]
                    }
                }, callback)
            }
        }
    }


    fun sendCommand2(byteArray: ByteArray) {
        sendCommand2(byteArray, null)
    }

    fun sendCommand2(byteArray: ByteArray, callback: Operation.DataSyncOperationCallback?) {
        enqueueOperation(
            byteArray,
            BleConstants.DEVICE_S_DATA_SERVICE_UUID,
            BleConstants.DEVICE_S_WRITE_CHARACTERISTIC_UUID,
            callback
        )
    }

    private fun enqueueOperation(
        byteArray: ByteArray,
        serviceUUID: UUID,
        uuid: UUID,
        callback: Operation.DataSyncOperationCallback?
    ) {
        enqueueOperation(byteArray, serviceUUID, uuid, false, callback)
    }

    private fun enqueueOperation(
        byteArray: ByteArray,
        serviceUUID: UUID,
        uuid: UUID,
        cutQueue: Boolean,
        callback: Operation.DataSyncOperationCallback?
    ) {
        if (isRelease) return
        mainHandler.post {
            queue.enqueueOperation(Operation(when (uuid) {
                BleConstants.DEVICE_COMMAND_WRITE_CHARACTERISTIC_UUID -> Operation.TYPE_COMMAND
                BleConstants.DEVICE_STATUS_WRITE_CHARACTERISTIC_UUID -> Operation.TYPE_STATUS
                BleConstants.DEVICE_S_WRITE_CHARACTERISTIC_UUID -> Operation.TYPE_S_COMMAND
                else -> -1
            }, byteArray, {
                Log.d(
                    TAG,
                    "write uuid:$uuid ,value:${
                        Utils.byteArrayToHexStrWithSpaceSplit(byteArray)
                    }"
                )
                gatt.getService(serviceUUID)?.getCharacteristic(uuid)?.let {
                    it.value = byteArray
                    val success = gatt.writeCharacteristic(it)
                    Log.e(TAG, "enqueueOperation result: $success")
                    if (!success) {
                        mainHandler.postDelayed({
                            enqueueOperation(
                                byteArray,
                                serviceUUID,
                                uuid,
                                cutQueue,
                                callback
                            )
                            queue.currentOperationDone()
                        }, 100)

                    }
                }
            }, object : Operation.DataSyncOperationCallback {
                override fun onFail(status: Int) {
                    // TODO: 2021/9/19 通知失败
                    Log.d(
                        TAG,
                        "Operation Fail!: [${Utils.byteArrayToHexStrWithSpaceSplit(queue.currentOperation?.dataBytes)}] Reason:$status"
                    )

                    if (status == REMOTE_BUSY) {
                        printLog("data write fail remote busy!")
                        enqueueOperation(byteArray, serviceUUID, uuid, true, callback)
                    } else {
                        callback?.onFail(status)
                    }
                    queue.currentOperationDone()
                }

                override fun onSuccess() {
                    Log.d(
                        TAG,
                        "Operation Success: [${Utils.byteArrayToHexStrWithSpaceSplit(queue.currentOperation?.dataBytes)}]"
                    )
                    callback?.onSuccess()
                    queue.currentOperationDone()
                }
            }), cutQueue
            )
            queue.runNext()
        }
    }

    private fun decodeBuffer(buffer: ByteBuffer, end: Boolean): List<String> {
        buffer.flip()
        val fileListBytes = ByteArray(buffer.limit())
        val size = buffer.get(fileListBytes)
        buffer.compact()
        val fileNamesStr = String(fileListBytes)
        val fileArray = fileNamesStr.split("\n")
        if (end.not()) {
            //不是结束
            if (fileArray.size == 0) {
                //全部没解出来
            } else {
                buffer.put(fileArray[fileArray.size - 1].toByteArray())
            }
        }
        return fileArray
    }

    /**
     * 收到realtek发来的log输出命令
     *
     * @param datas
     */
    fun dealLogReceived(datas: ByteArray, mac: String) {
        var datas = datas
        var hasEnd = false
        var index = 0
        for (data in datas) {
            if (data == 0x00.toByte()) {
                //收到结束标志位
                datas = Arrays.copyOfRange(datas, 0, index)
                hasEnd = true
                break
            }
            index++
        }
        byteBuffer?.let {
            it.put(datas)
            hasEnd = hasEnd or (datas.size < 19)
            if (hasEnd) {
                initLogHelper(mac)
                it.flip()
                if (it.hasRemaining().not()) return
                val bytes = ByteArray(it.remaining())
                it.get(bytes)
                it.compact()
                val content = String(bytes)
                printLog(content)
                val log = String.format(
                    "[DeviceLog --> %s /// deviceMac:%s] %s",
                    Utils.getFormatDateTimeMsStr(
                        System.currentTimeMillis(),
                        "yyyy-MM-dd HH:mm:ss SSS"
                    ),
                    mac,
                    content
                )
                logHelper?.saveStringLineData(logFileName + ".txt", log)
                Log.d(TAG, log)
            }
        }
    }

    private fun initLogHelper(mac: String) {
        if (Utils.isSameDay(logDateTime, Utils.getCurrentTimeInSecond())) {
            return
        }
        logDateTime = Utils.getCurrentTimeInSecond()
        logFileName =
            String.format("%s_%s", mac, Utils.getFormatDateTimeYMDStr(context, logDateTime))
        try {
            logHelper = FileSaveHelper("${context.getExternalFilesDir(null)}/data/", mac)
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        }
    }

    private fun printLog(msg: String) {
        logCallback?.onLog(msg)
    }

    override fun onOperationTimeout(operationCode: String?) {
        Log.d(TAG, "onOperationTimeout code:$operationCode")
    }

    override fun onOperationRun(progress: Int) {
    }

    override fun onAllOperationFinish() {
    }

    inner class FileSaveTransfer(
        var offset: Long,
        var fileName: String,
        var beginTime: Long,
        var rangeBeginTime: Long,
        var rangeEndTime: Long,
        var saveHelper: FileSaveHelper,
        var fileDataBuffer: ByteBuffer,
    ) {
        var crcErrorTimes: Int = 0

        override fun toString(): String {
            return "FileSaveTransfer(offset=$offset, fileName='$fileName', beginTime=$beginTime, rangeBeginTime=$rangeBeginTime, rangeEndTime=$rangeEndTime, saveHelper=$saveHelper, fileDataBuffer=$fileDataBuffer, crcErrorTimes=$crcErrorTimes)"
        }
    }

    inner class IMUTransfer(
        var beginTime: Long,
        var accRange: Int,
        var gyroRange: Int,
        var geoRange: Int,
        var rate: Int,
        var imuDataSaveHelper: FileSaveHelper
    ) {
        override fun toString(): String {
            return "IMUTransfer(beginTime=$beginTime, accRange=$accRange, gyroRange=$gyroRange, geoRange=$geoRange, rate=$rate, imuDataSaveHelper=$imuDataSaveHelper)"
        }
    }


    inner class RTHTransfer(var beginTime: Long, var dataSaveHelper: FileSaveHelper) {
        override fun toString(): String {
            return "RTHTransfer(beginTime=$beginTime, dataSaveHelper=$dataSaveHelper)"
        }
    }
}