package com.ltdigital.sdk.android.demo.bluetooth;

import android.os.Handler;
import android.os.Looper;

import com.android.core.Utils;
import com.android.core.log.Log;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

public class OperationQueue {

    private final String TAG = "OperationQueue";

    private Handler handler;
    private Operation mCurrentOperation;
    private List<Operation> mOperationQueue = new LinkedList<>();
    private OperationCallback callback;
    private int totalOperation;
    private long operationTimeoutMs = 5000;

    public OperationQueue(OperationCallback callback) {
        Looper looper = Looper.myLooper();
        if (looper == null) {
            Looper.prepare();
        }
        handler = new Handler();
        this.callback = callback;
    }

    public synchronized void currentOperationDone() {
        if (mCurrentOperation != null)
            mCurrentOperation.done();
        runNext();
    }

    public synchronized void currentOperationDelay() {
        handler.removeCallbacksAndMessages(null);
        handler.postDelayed(new checkTimeoutRunnable(), operationTimeoutMs);
    }

    public synchronized void enqueueOperation(Operation operation) {
        enqueueOperation(operation, false);
    }

    public synchronized void enqueueOperation(Operation operation, boolean needCutQueue) {
        if (mOperationQueue.size() >= 100) {
            //the queue is full
            return;
        }
        if (needCutQueue) {
            mOperationQueue.add(0, operation);
        } else {
            mOperationQueue.add(operation);
        }
        totalOperation++;
    }

    public synchronized boolean checkOperationAlreadyExist(byte[] dataBytes) {
        if (mCurrentOperation != null) {
            if (Arrays.equals(mCurrentOperation.dataBytes, dataBytes)) {
                return true;
            }
        }
        for (Operation syncOperation : mOperationQueue) {
            if (syncOperation == null) continue;
            if (Arrays.equals(syncOperation.dataBytes, dataBytes)) {
                return true;
            }
        }
        return false;
    }

    public synchronized Operation getCurrentOperation() {
        return mCurrentOperation;
    }

    public synchronized void removeAllOperations() {
        if (handler != null) {
            handler.removeCallbacksAndMessages(null);
        }
        boolean currentHaveOperation = mOperationQueue.size() != 0 || mCurrentOperation != null;
        mOperationQueue.clear();
        if (mCurrentOperation != null) {
            mCurrentOperation.done();
            mCurrentOperation = null;
        }
        if (currentHaveOperation && callback != null) {
            callback.onAllOperationFinish();
        }
    }

    public synchronized void runNext() {
        if (mCurrentOperation == null) {
            if (mOperationQueue.size() != 0) {
                mCurrentOperation = mOperationQueue.get(0);
                mOperationQueue.remove(0);
            }

            if (mCurrentOperation != null) {
                mCurrentOperation.run();
                handler.postDelayed(new checkTimeoutRunnable(), operationTimeoutMs);
            } else {
                if (callback != null) {
                    callback.onAllOperationFinish();
                }
                totalOperation = 0;
            }
            return;
        }

        if (mCurrentOperation.isDone()) {
            mCurrentOperation = null;
            if (callback != null) {
                int progress = 0;
                if (totalOperation != 0) {
                    int operationLeft = mOperationQueue.size();
                    progress = (totalOperation - operationLeft) * 100 / totalOperation;
                }
                callback.onOperationRun(progress);
            }
            runNext();
        }
    }

    private class checkTimeoutRunnable implements Runnable {
        @Override
        public void run() {
            synchronized (OperationQueue.this) {
                if (mCurrentOperation != null && System.currentTimeMillis() - mCurrentOperation.startRunTime >= (operationTimeoutMs - 100)) {
                    //超时时间范围内还没执行完
                    Log.d(TAG, "operationTimeOut -- code:" + Utils.byteArrayToHexStrWithSpaceSplit(mCurrentOperation.dataBytes));
                    if (callback != null) {
                        callback.onOperationTimeout(Utils.byteArrayToHexStrWithSpaceSplit(mCurrentOperation.dataBytes));
                    }
                    mCurrentOperation.done();
                    if (mCurrentOperation.callback != null)
                        mCurrentOperation.callback.onFail(Operation.DataSyncOperationCallback.STATUS_TIME_OUT);
                    runNext();
                }
            }
        }
    }

    public boolean isAllOperationFinish() {
        return mOperationQueue.size() == 0 && (mCurrentOperation == null || mCurrentOperation.isDone());
    }

    public interface OperationCallback {
        void onOperationTimeout(String operationCode);

        void onOperationRun(int progress);

        void onAllOperationFinish();
    }

}